*(c) Copyright 1993, 1994  Diane Merriam  All Rights Reserved

#include "pw.ch"          //Main header file

/*
This program includes the functions to allow a user to set all aspects of his
colors as well as several supporting routines. I hope I've documented them well
enough, but if there's any questions I can be reached on CIS at 76676,1663. If
you find them useful (and they save you time and/or make your clients happy), I
wouldn't sneeze at a few dollars sent my way.

Diane Merriam
115 Hillcrest Ave.
Louisville, KY 40206

May 19, 1994
*/

*The following defines are usually in a separate .CH, but I've put them all
*here to keep everything in one file

#xcommand DEFAULT <param1> TO <value1> [, <param2> TO <value2> ] =>     ;
         <param1> := IF(<param1> == NIL, <value1>, <param1>)            ;
         [; <param2> := IF(<param2> == NIL, <value2>, <param2>) ]

#xcommand TYPEDEF <param1> IS <type1> TO <value1>;
               [, <param2> IS <type2> TO <value2> ] =>     ;
         <param1> := IF(!valtype(<param1>) $ <"type1">, <value1>, <param1>);
      [; <param2> := IF(!valtype(<param2>) $ <"type2">, <value2>, <param2>)]

#xcommand REPEAT => DO WHILE .T.
#xcommand UNTIL <expl> => IF (<expl>); EXIT; END; END

#xcommand ASHRINK( < arr > ) => asize( < arr >, len( < arr > ) - 1)

#xtranslate MAKEBLOK( < func > ) => &( "{||" + alltrim( < func > ) + "}" )

#xtranslate STRTRIM(<num>, <len>, <dec>) => ltrim(str(<num>, <len>, <dec>))
#xtranslate STRPADL(<num>, <len>, <dec>) =>;
                            padl(ltrim(str(<num>, <len> - 1, <dec>)), <len>)
#xtranslate STRPADR(<num>, <len>, <dec>) =>;
                            padr(ltrim(str(<num>, <len> - 1, <dec>)), <len>)

#ifndef TRUE
#define   TRUE    .T.
#endif

#ifndef FALSE
#define   FALSE   .F.
#endif


*Arrays of possible colors for foreground and background
static aFore := {"N",   "B",   "G",   "BG",   "R",   "RB",   "GR",   "W",;
                     "+N",  "+B",  "+G",  "+BG",  "+R",  "+RB",  "+GR",  "+W"}
static aBack := {"N",   "B",   "G",   "BG",   "R",   "RB",   "GR",   "W",;
                     "*N",  "*B",  "*G",  "*BG",  "*R",  "*RB",  "*GR",  "*W"}
*Initial settings

static nFore := 1           //Pointer into aFore
static nBack := 8           //Pointer into aBack
static lStart := .F.        //Color choose ok button value
static cCurColor := "N/W"   //String of currently active color being changed
static oChoose              //Color choice window
static aColor               //Array of individual colorscheme values
static oColor               //Starting color scheme object

function W_PwColor()
/*
This function creates the main window for changing the colors.
*/
local oWin, oMenu, oSelect, oOriginal

W_Wait(.T., "Creating color choice window")

*Get the current color scheme
oOriginal := pw():getColorScheme()

*Create the window with cleanup close handler
CREATE WINDOW oWin;
  AT 0, 0;
  SIZE pwMaxRow(), pwMaxCol();
  STYLE PWSTYLE_DIALOG + PWSTYLEINFO_HELPLINE + PWSTYLEINFO_MENU;
  TITLE "Color Settings";
  ICON TITLE "Color";
  WINDOW HANDLER {|idMsg|if(idMsg == PWMSG_CLOSE,;
    (if(valtype(oChoose) == "O", oChoose:destroy(), ), oWin:destroy()), ), .T.};
  HELPLINE "Select Color"

/*
This menu allows every possible color setting to be changed. It also allows for
a return to default, cancel changes, save changes, and sample view.
The W_ColorWin function is the selector of new colors

The numeric references refer to the elements in the colorscheme array as
follows:

row  Type
-----------------
1    BrowseBox
2    CheckBox
3    CheckList
4    ComboBox
5    ListBox
6    Menus
7    MenuItems
8    Notepad
9    Popups
10   PushButton
11   RadioButton
12   Spinner
13   SystemMenu
14   TextField
15   Window

The individual elements of each row are documented in the .NG under
Miscellaneous/ColorScheme Class.
*/

MENU oMenu
  POPUP PROMPT "~Windows";
    HELPLINE "Set the colors for regular and notepad windows."

    POPUP PROMPT "Standard Windows";
      HELPLINE "Set the colors for the regular screen windows."

      MENUITEM PROMPT "Inside Color";
        HELPLINE "Color to use for normal text inside the window.";
        ACTION {||W_ColorWin(15, 1)}

      MENUITEM PROMPT "Unselected Border Color";
        HELPLINE "Color to use for the border when the window is unselected.";
        ACTION {||W_ColorWin(15, 2)}

      MENUITEM PROMPT "Selected Border Color";
        HELPLINE "Color to use for the border when the window is selected.";
        ACTION {||W_ColorWin(15, 3)}

      MENUITEM PROMPT "Scrollbar Color";
        HELPLINE "Color to use for the scrollbar (if any).";
        ACTION {||W_ColorWin(15, 4)}

      MENUITEM PROMPT "Menu Line Color";
        HELPLINE "Color to use for the menu line.";
        ACTION {||W_ColorWin(15, 5)}

      MENUITEM PROMPT "Help Line Color";
        HELPLINE "Color to use for the help line.";
        ACTION {||W_ColorWin(15, 6)}
    END POPUP

    POPUP PROMPT "Notepad Windows";
      HELPLINE "Set the colors for a notepad (freeform text entry) windows."

      MENUITEM PROMPT "Inside Color";
        HELPLINE "Color to use for normal text inside the window.";
        ACTION {||W_ColorWin(8, 1)}

      MENUITEM PROMPT "Unselected Border Color";
        HELPLINE "Color to use for the border when the window is unselected.";
        ACTION {||W_ColorWin(8, 2)}

      MENUITEM PROMPT "Selected Border Color";
        HELPLINE "Color to use for the border when the window is selected.";
        ACTION {||W_ColorWin(8, 3)}

      MENUITEM PROMPT "Scrollbar Color";
        HELPLINE "Color to use for the scrollbar (if any).";
        ACTION {||W_ColorWin(8, 4)}

      MENUITEM PROMPT "Menu Line Color";
        HELPLINE "Color to use for the menu line.";
        ACTION {||W_ColorWin(8, 5)}

      MENUITEM PROMPT "Help Line Color";
        HELPLINE "Color to use for the help line.";
        ACTION {||W_ColorWin(8, 6)}
    END POPUP
  END POPUP

  POPUP PROMPT "~Menus";
    HELPLINE "Set the colors for the various levels of menus."

    POPUP PROMPT "Bar Menu";
      HELPLINE "Set the colors for the top level horizontal bar menus."

      MENUITEM PROMPT "Unselected Color";
        HELPLINE "Color to use for unselected enabled menu items.";
        ACTION {||W_ColorWin(6, 1)}

      MENUITEM PROMPT "Selected Color";
        HELPLINE "Color to use for the currently selected enabled menu item.";
        ACTION {||W_ColorWin(6, 2)}

      MENUITEM PROMPT "Shortcut Key Color";
        HELPLINE "Color to use for highlighting the shortcut key.";
        ACTION {||W_ColorWin(6, 3)}

      MENUITEM PROMPT "Disabled Item Color";
        HELPLINE "Color to use for disabled items.";
        ACTION {||W_ColorWin(6, 4)}

      MENUITEM PROMPT "Disabled Item Select Color";
        HELPLINE "Color to use when user attempts to select a disabled item.";
        ACTION {||W_ColorWin(6, 5)}
    END POPUP

    POPUP PROMPT "Drop Menu";
      HELPLINE "Set the colors for the menus which drop down from the bar."

      MENUITEM PROMPT "Unselected Color";
        HELPLINE "Color to use for unselected enabled menu items.";
        ACTION {||W_ColorWin(7, 1)}

      MENUITEM PROMPT "Selected Color";
        HELPLINE "Color to use for the currently selected enabled menu item.";
        ACTION {||W_ColorWin(7, 2)}

      MENUITEM PROMPT "Shortcut Key Color";
        HELPLINE "Color to use for highlighting the shortcut key.";
        ACTION {||W_ColorWin(7, 3)}

      MENUITEM PROMPT "Disabled Item Color";
        HELPLINE "Color to use for disabled items.";
        ACTION {||W_ColorWin(7, 4)}

      MENUITEM PROMPT "Disabled Item Select Color";
        HELPLINE "Color to use when user attempts to select a disabled item.";
        ACTION {||W_ColorWin(7, 5)}
    END POPUP

    POPUP PROMPT "Popup Menu";
      HELPLINE "Set the colors for the third level or lower menus."

      MENUITEM PROMPT "Unselected Color";
        HELPLINE "Color to use for unselected enabled menu items.";
        ACTION {||W_ColorWin(9, 1)}

      MENUITEM PROMPT "Selected Color";
        HELPLINE "Color to use for the currently selected enabled menu item.";
        ACTION {||W_ColorWin(9, 2)}

      MENUITEM PROMPT "Shortcut Key Color";
        HELPLINE "Color to use for highlighting the shortcut key.";
        ACTION {||W_ColorWin(9, 3)}

      MENUITEM PROMPT "Disabled Item Color";
        HELPLINE "Color to use for disabled items.";
        ACTION {||W_ColorWin(9, 4)}

      MENUITEM PROMPT "Disabled Item Select Color";
        HELPLINE "Color to use when user attempts to select a disabled item.";
        ACTION {||W_ColorWin(9, 5)}
    END POPUP

    POPUP PROMPT "System Menu";
      HELPLINE "Set the colors for the system menus attached to windows."

      MENUITEM PROMPT "Unselected Color";
        HELPLINE "Color to use for unselected enabled menu items.";
        ACTION {||W_ColorWin(13, 1)}

      MENUITEM PROMPT "Selected Color";
        HELPLINE "Color to use for the currently selected enabled menu item.";
        ACTION {||W_ColorWin(13, 2)}

      MENUITEM PROMPT "Shortcut Key Color";
        HELPLINE "Color to use for highlighting the shortcut key.";
        ACTION {||W_ColorWin(13, 3)}

      MENUITEM PROMPT "Disabled Item Color";
        HELPLINE "Color to use for disabled items.";
        ACTION {||W_ColorWin(13, 4)}

      MENUITEM PROMPT "Disabled Item Select Color";
        HELPLINE "Color to use when user attempts to select a disabled item.";
        ACTION {||W_ColorWin(13, 5)}
    END POPUP
  END POPUP

  POPUP PROMPT "~Browses";
    HELPLINE "Set the colors for scrolling data type areas."

    POPUP PROMPT "Data Browse";
      HELPLINE "Set the colors for normal database and array browses."

      MENUITEM PROMPT "Unselected Color";
        HELPLINE "Color to use for the unselected rows or cells.";
        ACTION {||W_ColorWin(1, 1)}

      MENUITEM PROMPT "Selected Color";
        HELPLINE "Color to use for the selected row or cell.";
        ACTION {||W_ColorWin(1, 2)}

      MENUITEM PROMPT "Prompt Color";
        HELPLINE "Color to use for the box border and prompt (if any).";
        ACTION {||W_ColorWin(1, 3)}

      MENUITEM PROMPT "Shortcut Key Color";
        HELPLINE "Color to use to highlight the shortcut key (if any).";
        ACTION {||W_ColorWin(1, 4)}

      MENUITEM PROMPT "Scrollbar Border Color";
        HELPLINE "Color to use to draw the scrollbar border.";
        ACTION {||W_ColorWin(1, 5)}

      MENUITEM PROMPT "Scrollbar Color";
        HELPLINE "Color to use for the scrollbar(s).";
        ACTION {||W_ColorWin(1, 6)}

      MENUITEM PROMPT "Disabled Color";
        HELPLINE "Color to use for drawing the boxes when disabled.";
        ACTION {||W_ColorWin(1, 7)}
    END POPUP

    POPUP PROMPT "Check List";
      HELPLINE "Set the colors for a checklist type area"

      MENUITEM PROMPT "Unselected Color";
        HELPLINE "Color to use for the unselected rows or cells.";
        ACTION {||W_ColorWin(3, 1)}

      MENUITEM PROMPT "Selected Color";
        HELPLINE "Color to use for the selected row or cell.";
        ACTION {||W_ColorWin(3, 2)}

      MENUITEM PROMPT "Prompt Color";
        HELPLINE "Color to use for the box border and prompt (if any).";
        ACTION {||W_ColorWin(3, 3)}

      MENUITEM PROMPT "Shortcut Key Color";
        HELPLINE "Color to use to highlight the shortcut key (if any).";
        ACTION {||W_ColorWin(3, 4)}

      MENUITEM PROMPT "Scrollbar Border Color";
        HELPLINE "Color to use to draw the scrollbar border.";
        ACTION {||W_ColorWin(3, 5)}

      MENUITEM PROMPT "Scrollbar Color";
        HELPLINE "Color to use for the scrollbar(s).";
        ACTION {||W_ColorWin(3, 6)}

      MENUITEM PROMPT "Disabled Color";
        HELPLINE "Color to use for drawing the boxes when disabled.";
        ACTION {||W_ColorWin(3, 7)}
    END POPUP

    POPUP PROMPT "List Box";
      HELPLINE "Set the colors for a quick list box."

      MENUITEM PROMPT "Unselected Color";
        HELPLINE "Color to use for the unselected rows or cells.";
        ACTION {||W_ColorWin(5, 1)}

      MENUITEM PROMPT "Selected Color";
        HELPLINE "Color to use for the selected row or cell.";
        ACTION {||W_ColorWin(5, 2)}

      MENUITEM PROMPT "Prompt Color";
        HELPLINE "Color to use for the box border and prompt (if any).";
        ACTION {||W_ColorWin(5, 3)}

      MENUITEM PROMPT "Shortcut Key Color";
        HELPLINE "Color to use to highlight the shortcut key (if any).";
        ACTION {||W_ColorWin(5, 4)}

      MENUITEM PROMPT "Scrollbar Border Color";
        HELPLINE "Color to use to draw the scrollbar border.";
        ACTION {||W_ColorWin(5, 5)}

      MENUITEM PROMPT "Scrollbar Color";
        HELPLINE "Color to use for the scrollbar(s).";
        ACTION {||W_ColorWin(5, 6)}

      MENUITEM PROMPT "Disabled Color";
        HELPLINE "Color to use for drawing the boxes when disabled.";
        ACTION {||W_ColorWin(5, 7)}
    END POPUP

    POPUP PROMPT "Combo Box";
      HELPLINE "Set the colors for a drop down quick list box."

      MENUITEM PROMPT "Unselected Color";
        HELPLINE "Color to use for the unselected rows or cells.";
        ACTION {||W_ColorWin(4, 1)}

      MENUITEM PROMPT "Selected Color";
        HELPLINE "Color to use for the selected row or cell.";
        ACTION {||W_ColorWin(4, 2)}

      MENUITEM PROMPT "Prompt Color";
        HELPLINE "Color to use for the box border and prompt (if any).";
        ACTION {||W_ColorWin(4, 3)}

      MENUITEM PROMPT "Shortcut Key Color";
        HELPLINE "Color to use to highlight the shortcut key (if any).";
        ACTION {||W_ColorWin(4, 4)}

      MENUITEM PROMPT "Scrollbar Border Color";
        HELPLINE "Color to use to draw the scrollbar border.";
        ACTION {||W_ColorWin(4, 5)}

      MENUITEM PROMPT "Scrollbar Color";
        HELPLINE "Color to use for the scrollbar(s).";
        ACTION {||W_ColorWin(4, 6)}

      MENUITEM PROMPT "Disabled Color";
        HELPLINE "Color to use for drawing the boxes when disabled.";
        ACTION {||W_ColorWin(4, 7)}
    END POPUP
  END POPUP

  POPUP PROMPT "~Controls";
    HELPLINE "Set the colors for the various types of input control areas."

    POPUP PROMPT "Standard input";
      HELPLINE "Set the colors for regular typed field data entry."

      MENUITEM PROMPT "Unselected Color";
        HELPLINE "Color to use for this control when unselected.";
        ACTION {||W_ColorWin(14, 1)}

      MENUITEM PROMPT "Selected Color";
        HELPLINE "Color to use for this control when selected.";
        ACTION {||W_ColorWin(14, 2)}

      MENUITEM PROMPT "Shortcut Key Color";
        HELPLINE "Color to use for the prompt shortcut key (if any).";
        ACTION {||W_ColorWin(14, 3)}

      MENUITEM PROMPT "Disabled Color";
        HELPLINE "Color to use for this control if disabled.";
        ACTION {||W_ColorWin(14, 4)}
    END POPUP

    POPUP PROMPT "Check Box";
      HELPLINE "Set the colors for a logical (yes/no) check box."

      MENUITEM PROMPT "Unselected Color";
        HELPLINE "Color to use for this control when unselected.";
        ACTION {||W_ColorWin(2, 1)}

      MENUITEM PROMPT "Selected Color";
        HELPLINE "Color to use for this control when selected.";
        ACTION {||W_ColorWin(2, 2)}

      MENUITEM PROMPT "Shortcut Key Color";
        HELPLINE "Color to use for the prompt shortcut key (if any).";
        ACTION {||W_ColorWin(2, 3)}

      MENUITEM PROMPT "Disabled Color";
        HELPLINE "Color to use for this control if disabled.";
        ACTION {||W_ColorWin(2, 4)}
    END POPUP

    POPUP PROMPT "Radio Buttons";
      HELPLINE "Set the colors for a set of radio buttons (pick one of)"

      MENUITEM PROMPT "Unselected Color";
        HELPLINE "Color to use for this control when unselected.";
        ACTION {||W_ColorWin(11, 1)}

      MENUITEM PROMPT "Selected Color";
        HELPLINE "Color to use for this control when selected.";
        ACTION {||W_ColorWin(11, 2)}

      MENUITEM PROMPT "Shortcut Key Color";
        HELPLINE "Color to use for the prompt shortcut key (if any).";
        ACTION {||W_ColorWin(11, 3)}

      MENUITEM PROMPT "Disabled Color";
        HELPLINE "Color to use for this control if disabled.";
        ACTION {||W_ColorWin(11, 4)}
    END POPUP

    POPUP PROMPT "Spinners";
      HELPLINE "Set the colors for spinners on numeric or date entry."

      MENUITEM PROMPT "Unselected Color";
        HELPLINE "Color to use for this control when unselected.";
        ACTION {||W_ColorWin(12, 1)}

      MENUITEM PROMPT "Selected Color";
        HELPLINE "Color to use for this control when selected.";
        ACTION {||W_ColorWin(12, 2)}

      MENUITEM PROMPT "Shortcut Key Color";
        HELPLINE "Color to use for the prompt shortcut key (if any).";
        ACTION {||W_ColorWin(12, 3)}

      MENUITEM PROMPT "Disabled Color";
        HELPLINE "Color to use for this control if disabled.";
        ACTION {||W_ColorWin(12, 4)}
    END POPUP
  END POPUP

  POPUP PROMPT "~Push Buttons";
    HELPLINE "Set the colors for push buttons."

    MENUITEM PROMPT "Unselected Color";
      HELPLINE "Color to use for this control when unselected.";
      ACTION {||W_ColorWin(10, 1)}

    MENUITEM PROMPT "Selected Color";
      HELPLINE "Color to use for this control when selected.";
      ACTION {||W_ColorWin(10, 2)}

    MENUITEM PROMPT "Shortcut Key Color";
      HELPLINE "Color to use for the shortcut key highlight.";
      ACTION {||W_ColorWin(10, 3)}

    MENUITEM PROMPT "Bright Color";
      HELPLINE "Color to use for top border.";
      ACTION {||W_ColorWin(10, 4)}

    MENUITEM PROMPT "Dim Color";
      HELPLINE "Color to use for bottom border.";
      ACTION {||W_ColorWin(10, 5)}

    MENUITEM PROMPT "Default Color";
      HELPLINE "Color to use for .";
      ACTION {||W_ColorWin(10, 6)}

    MENUITEM PROMPT "Disabled Color";
      HELPLINE "Color to use when this control is disabled.";
      ACTION {||W_ColorWin(10, 7)}
  END POPUP

  MENUITEM PROMPT "~Default";
    HELPLINE "Return to default colors";
    ACTION {||pw():setColorScheme(oOriginal:init()), oWin:Close()}

  MENUITEM PROMPT "~Cancel";
    HELPLINE "Abort changes and return to main menu.";
    ACTION {||pw():setColorScheme(oOriginal), oWin:Close()}

  MENUITEM PROMPT "~Finished";
    HELPLINE "Save changes and return to main menu.";
    ACTION {||W_SaveColr(), oWin:Close()}

  MENUITEM PROMPT "~Sample";
    HELPLINE "See a sample window with your new color selections.";
    ACTION {||MakeSample()}
END MENU

ATTACH MENU oMenu TO oWin

W_Wait(.F.)

OPEN WINDOW oWin

return NIL


function W_ColorWin(nType, nElement)
/*
This function simply sets up a pair of radiobuttons for selecting the new color
from and displays a string in the color as it changes.

nType is the color array row (see row descriptions in the documentation for
W_PwColor). nElement is the individual element in the row as described in the
.NG
*/
local oOk, oCancel, oFore, oBack, oBlink, aList, nLoop
local getList := {}

W_Wait(.T., "Creating color choice window")

*Set up the window if it's not already done.
if ! valtype(oChoose) == "O"

*Get the current color scheme to allow for cancellation
  oColor := pw():getColorScheme()

*Get the current colors into an array that can be worked with directly
  aColor := W_Colors(.T.)

*Set the array pointers (nFore and nBack) into aColor as well as setting the
*string eqivalent (cCurColor)
  NewColor(aColor[nType, nElement])

  CREATE WINDOW oChoose CENTERED;
    SIZE 23, 42;
    STYLE PWSTYLE_DIALOG + PWSTYLEINFO_HELPLINE;
    TITLE "Color Selection";
    HELPLINE "Color Selection";
    ICON TITLE "Colors"

*These radio buttons select the new color, and upon change, display text in the
*selected color
  @ 1, 1 GET nFore AS RADIOBUTTONS oFore;
    PROMPT {"Black",;
            "Blue",;
            "Green",;
            "Cyan",;
            "Red",;
            "Magenta",;
            "Brown",;
            "White",;
            "Grey",;
            "Bright Blue",;
            "Bright Green",;
            "Bright Cyan",;
            "Bright Red",;
            "Bright Magenta",;
            "Bright Yellow",;
            "Bright White"};
    HELPLINE "Foreground Color";
    ACTION {||DispLine(oFore, oBack)}

  @ 1, 22 GET nBack AS RADIOBUTTONS oBack;
    PROMPT {"Black",;
            "Blue",;
            "Green",;
            "Cyan",;
            "Red",;
            "Magenta",;
            "Brown",;
            "White",;
            "Grey",;
            "Bright Blue",;
            "Bright Green",;
            "Bright Cyan",;
            "Bright Red",;
            "Bright Magenta",;
            "Bright Yellow",;
            "Bright White"};
    HELPLINE "Background Color";
    ACTION {||DispLine(oFore, oBack)}

*UpColor modifies the ColorScheme class to the new color chosen
  @ 17, 1 GET lStart AS PUSHBUTTON oOk;
    PROMPT " ~Ok ";
    ACTION {|| UpColor(nType, nElement), oChoose:close() }

  @ 17, 11 GET lStart AS PUSHBUTTON oCancel;
    PROMPT " ~Cancel ";
    ACTION {|| oChoose:close()}

  ATTACH CONTROLS TO oChoose

else

*Set the array pointers (nFore and nBack) into aColor as well as setting the
*string eqivalent (cCurColor)
  NewColor(aColor[nType, nElement])

*Set the radiobuttons to the current color that we want to change from
  aList := oChoose:getGetlist()
  for nLoop := 1 to 2
    aList[nLoop]:setValue(eval(aList[nLoop]:varBlock))
  next
  aList[3]:setvalue(.F.)
  aList[4]:setvalue(.F.)
endif

W_Wait(.F.)

OPEN WINDOW oChoose

*Make SURE we're there
pw():select(oChoose)

*Sample text in the current color
@ 18, 25 say "Sample Color" color cCurColor

return oChoose


static function DispLine(oFore, oBack)
/*
Update the sample text line in the color choice select window.
oFore and oBack are the radiobutton controls
*/

*For display of the sample text in the color select window
@ 18, 25 say "Sample Color" color (aFore[oFore:getValue()] +;
                                                 "/" + aBack[oBack:getValue()])
return .F.




static function MakeSample()
/*
This simply allows the user to see all elements of the colors they've chosen in
the context of the others...IOW how well the various colors work together. We
set up menus that do nothing (except for exiting) and sample controls of all
types.
*/
local oDummy, oMenu, oBrowse, aBrowse, oBControl
local getlist := {}

W_Wait(.T., "Creating sample window")

aBrowse := {{"Select 1", "Select 2", "Select 3", "Select 4"},;
            {"Select A", "Select B", "Select C", "Select D"},;
            {"Info 1", "Info 2", "Info 3", "Info 4"}}

oBrowse := tbrowsenew(0, 0, 4, 11)
oBrowse:cargo := 1
oBrowse:addcolumn(tbcolumnnew("Column 1", {||aBrowse[1, oBrowse:cargo]}))
oBrowse:addcolumn(tbcolumnnew("Column 2", {||aBrowse[2, oBrowse:cargo]}))
oBrowse:addcolumn(tbcolumnnew("Column 3", {||aBrowse[3, oBrowse:cargo]}))
oBrowse:goTopBlock := {||oBrowse:cargo := 1}
oBrowse:goBottomBlock := {||oBrowse:cargo := 4}
oBrowse:skipBlock := {|n, nOld|nOld := oBrowse:cargo,;
  oBrowse:cargo := oBrowse:cargo + n, oBrowse:cargo := min(oBrowse:cargo, 4),;
            oBrowse:cargo := max(oBrowse:cargo, 1), n := oBrowse:cargo - nOld}
oBrowse:headSep := ""

CREATE WINDOW oDummy CENTERED;
  SIZE 25, 75;
  STYLE PWSTYLE_PRIMARY + PWSTYLEINFO_HELPLINE;
  TITLE "Sample window";
  HELPLINE "View effects of color changes.";
  WINDOW HANDLER {|idMsg| if(idMsg == PWMSG_CLOSE, pw():endModal(), ), .T.};
  SHADOW

MENU oMenu
  POPUP PROMPT "~Drop";
    HELPLINE "This is a drop menu"

    MENUITEM PROMPT "~Active 1";
      HELPLINE "This is an active drop menuitem";
      ACTION {||NIL}

    MENUITEM PROMPT "A~ctive 2";
      HELPLINE "This is an active drop menuitem";
      ACTION {||NIL}

    MENUITEM PROMPT "~Inactive";
      HELPLINE "This is an inactive drop menuitem";
      ACTION {||NIL};
      DISABLED

    POPUP PROMPT "~Popup";
      HELPLINE "This is a popup menu"

      MENUITEM PROMPT "~Active 1";
        HELPLINE "This is an active popup menuitem";
        ACTION {||NIL}

      MENUITEM PROMPT "A~ctive 2";
        HELPLINE "This is an active popup menuitem";
        ACTION {||NIL}

      MENUITEM PROMPT "~Inactive";
        HELPLINE "This is an inactive popup menuitem";
        ACTION {||NIL};
        DISABLED
    END POPUP
  END POPUP

  MENUITEM PROMPT "~Active 1";
    HELPLINE "This is an active bar menuitem";
    ACTION {||NIL}

  MENUITEM PROMPT "A~ctive 2";
    HELPLINE "This is an active bar menuitem";
    ACTION {||NIL}

  MENUITEM PROMPT "~Inactive";
    HELPLINE "This is an inactive bar menuitem";
    ACTION {||NIL};
    DISABLED

  MENUITEM PROMPT "~Cancel";
    HELPLINE "Return to color selection";
    ACTION {||oDummy:close()}
END MENU

ATTACH MENU oMenu TO oDummy

@ 1, 1 GET AS TEXTFIELD;
  INITIAL "Some information";
  PROMPT "~TextField: ";
  HELPLINE "This is a text field (standard data entry)"

@ 1, 32 GET AS SPINNER;
  INITIAL 10;
  RANGE 0, 99;
  PROMPT "~Spinner: ";
  PICTURE "99";
  HELPLINE "This is a spinner. It can be used for numeric or date entry."

@ 1, 55 GET AS CHECKBOX;
  INITIAL .T.;
  PROMPT "Check~box";
  HELPLINE "This is a checkbox. It is used for yes/no or true/false entry."

@ 3, 1 say "Radiobuttons (This is text)"

@ 4, 1 GET AS RADIOBUTTONS;
  INITIAL 2;
  PROMPT aBrowse[1];
  HELPLINE "These are radiobuttons. You may select only one of them."

@ 3, 32 GET AS COMBOBOX;
  USING aBrowse[2];
  INITIAL 1;
  PROMPT "~ComboBox: ";
  HELPLINE "This is a combobox. You may select only one of them."

@ 3, 55 GET AS LISTBOX;
  USING aBrowse[2];
  SIZE 6, 12;
  INITIAL 3;
  PROMPT "~Listbox";
  HELPLINE "This is a listbox. You may select only one of them."

@ 7, 18 GET AS BROWSEBOX oBControl;
  USING oBrowse;
  SIZE 8, 21;
  PROMPT "B~rowsebox";
  HELPLINE "This is a browsebox. You may select only one of them."

oBControl:setBrowseBlocks({|o, r, n|r := 4, n := o:cargo},;
                                                        {|o, r|o:cargo := r})

@ 9, 1 GET AS CHECKLIST;
  USING aBrowse[3];
  SIZE 6, 12;
  PROMPT "C~hecklist";
  HELPLINE "This is a checklist. You may check as many items as are available."

@ 16, 1 GET AS NOTEPAD;
  SIZE 4, 69;
  PROMPT "~Notepad";
  HELPLINE "This is a notepad. It is used for freeform text entry."

@ 22, 30 GET AS PUSHBUTTON;
  PROMPT "  ~Ok  ";
  HELPLINE "This is a pushbutton. It will cause an action to occur.";
  ACTION {||oDummy:close()}

ATTACH CONTROLS TO oDummy

W_Wait(.F.)

*Common handling for modal windows.
W_ModalEnd(oDummy)

return NIL



static function NewColor(cColor)
/*
Set the current selected color and update the statics
cColor must use the form +C or *C if the attributes are to be used
*/

*Remove all extraneous spaces
cCurColor := strtran(cColor, " ")

*Find the array position of the color forground and background
nFore := ascan(aFore, substr(cCurColor, 1, at("/", cCurColor) - 1))
nBack := ascan(aBack, substr(cCurColor, at("/", cCurColor) + 1))
return cCurColor




function W_SaveColr()
/*
Save a new color array with message and clean up
*/

W_Colors(.F., aColor)

*Previously created windows will not change colors
W_Message("All new colors will not be seen until the next time you start " +;
                                                                  "the system")
*Release the memory used by the array
aColor := {}

return NIL




static function UpColor(nType, nElement)
/*
Set the current color choice

nType is the color array row (see row descriptions in the documentation for
W_PwColor). nElement is the individual element in the row as described in the
.NG
*/
local cString

*Create the current color string from the values in the arrays
cCurColor := aFore[nFore] +  "/" + aBack[nBack]

*Update the colorscheme equivalent array
aColor[nType, nElement] := cCurColor

*Convert the array row into a single string
cString := W_Arr2Str(aColor[nType], ",")

*Set the appropriate ColorScheme object variable
do case
  case nType == 1
    oColor:BrowseBoxColor := cString
  case nType == 2
    oColor:CheckBoxColor := cString
  case nType == 3
    oColor:CheckListColor := cString
  case nType == 4
    oColor:ComboBoxColor := cString
  case nType == 5
    oColor:ListBoxColor := cString
  case nType == 6
    oColor:MenuColor := cString
  case nType == 7
    oColor:MenuItemColor := cString
  case nType == 8
    oColor:NotepadColor := cString
  case nType == 9
    oColor:PopupColor := cString
  case nType == 10
    oColor:PushbuttonColor := cString
  case nType == 11
    oColor:RadiobuttonColor := cString
  case nType == 12
    oColor:SpinnerColor := cString
  case nType == 13
    oColor:SystemMenuColor := cString
  case nType == 14
    oColor:TextFieldColor := cString
  case nType == 15
    oColor:WindowColor := cString
endcase

*Update the active ColorScheme
pw():setColorScheme(oColor)

return NIL


function W_ModalEnd(oWin, lDestroy, getlist, cColor)
/*
Standard handling for modal windows

oWin is the window to be opened modally (required)
lDestroy is whether or not to destroy the window on close
getlist is the current getlist if you want the function to add an ok button
cColor is an optional color to set the OK button with
*/
local nHeight, nWidth

DEFAULT lDestroy TO .T.

/*
Many modal windows have an "ok" button so this eliminates the need to
explicitly add it every time. It centers it on the bottom row
*/
if valtype(getlist) == "A"
  oWin:getSize(@nHeight, @nWidth)

  @ nHeight - 4, int(nWidth / 2) - 4 GET AS PUSHBUTTON;
    PROMPT "  ~OK  ";
    COLOR cColor;
    ACTION {||oWin:close()}

  ATTACH CONTROLS TO oWin
endif

*We don't need to bother adding this each time either
oWin:AddFilter({|idMsg|if(idMsg == PWMSG_CLOSE, pw():endModal(), NIL)},;
                                                                   PWMSG_CLOSE)
OPEN WINDOW oWin MODAL

*Make SURE we're here
pw():select(oWin)

*If we are, start the modal operation
if pw():curFocus() == oWin
  pw():beginModal()
endif

if lDestroy
  oWin:destroy()
endif

return NIL


function W_Colors(lWhich, aNew)
/*
This function is where you would interface with a permanent save routine. You
could store the color selection array in a memo field associated with a user
ID record or use one of the various array to .MEM routines to save it that way.

lWhich is what action to take (save or restore)
aNew is an optional new array of color values to save
*/
local oLocal, oColors

DEFAULT lWhich TO .T.
oColors := pw():getColorScheme()

if valtype(aNew) == "A"  //set the return array equal to the passed array
  aColor := aNew
endif

if lWhich  //Get the current settings
  if ! valtype(aColor) == "A"  //If not already set
/*
The following two lines are used with objectDb. I maintain a file on users
indexed on their logon name that also tracks the individual choices for things
such as color scheme, printer settings, etc.
If the user has never before changed their colors, there will be no array.
*/
*    oLocal  := W_GetLocal()
*    aColor := oLocal..SysColor
*    if ! valtype(aColor) == "A"     // Note the matched endif commented out

/*
Get the individual ColorScheme strings broken into array elements. Note that
some of the values in the ColorScheme may not have strings associated with them
originally. If this is the case, the default value is used just as is done in
PVW.
*/
      aColor := {}
      aadd(aColor, W_Str2Arr(oColors:BrowseBoxColor, ","))    //1
      aadd(aColor, W_Str2Arr(oColors:CheckBoxColor, ","))     //2
      aadd(aColor, W_Str2Arr(oColors:CheckListColor, ","))    //3
      aadd(aColor, W_Str2Arr(oColors:ComboBoxColor, ","))     //4
      aadd(aColor, W_Str2Arr(oColors:ListBoxColor, ","))      //5
      aadd(aColor, W_Str2Arr(oColors:MenuColor, ","))         //6
      if empty(oColors:MenuItemColor)                         //7
        aadd(aColor, aclone(aColor[6]))
      else
        aadd(aColor, W_Str2Arr(oColors:MenuItemColor, ","))
      endif
      aadd(aColor, W_Str2Arr(oColors:NotepadColor, ","))      //8
      if empty(oColors:PopupColor)                            //9
        aadd(aColor, aclone(aColor[6]))
      else
        aadd(aColor, W_Str2Arr(oColors:PopupColor, ","))
      endif
      aadd(aColor, W_Str2Arr(oColors:PushbuttonColor, ","))   //10
      aadd(aColor, W_Str2Arr(oColors:RadiobuttonColor, ","))  //11
      aadd(aColor, W_Str2Arr(oColors:SpinnerColor, ","))      //12
      aadd(aColor, W_Str2Arr(oColors:SystemMenuColor, ","))   //13
      aadd(aColor, W_Str2Arr(oColors:TextFieldColor, ","))    //14
      aadd(aColor, W_Str2Arr(oColors:WindowColor, ","))       //15
*    endif
  endif
/*
If you're not saving color schemes anywhere, this following section is useless.
It makes sure that the PVW ColorScheme is in accordance with the color array.
*/
  oColors:browseBoxColor :=    W_Arr2Str(aColor[1], ",")
  oColors:checkBoxColor :=     W_Arr2Str(aColor[2], ",")
  oColors:checkListColor :=    W_Arr2Str(aColor[3], ",")
  oColors:comboBoxColor :=     W_Arr2Str(aColor[4], ",")
  oColors:listBoxColor :=      W_Arr2Str(aColor[5], ",")
  oColors:menuColor :=         W_Arr2Str(aColor[6], ",")
  oColors:menuItemColor :=     W_Arr2Str(aColor[7], ",")
  oColors:notepadColor :=      W_Arr2Str(aColor[8], ",")
  oColors:popupColor :=        W_Arr2Str(aColor[9], ",")
  oColors:pushbuttonColor :=   W_Arr2Str(aColor[10], ",")
  oColors:radiobuttonsColor := W_Arr2Str(aColor[11], ",")
  oColors:spinnerColor :=      W_Arr2Str(aColor[12], ",")
  oColors:systemMenuColor :=   W_Arr2Str(aColor[13], ",")
  oColors:textfieldColor :=    W_Arr2Str(aColor[14], ",")
  oColors:windowColor :=       W_Arr2Str(aColor[15], ",")
  pw():setColorScheme(oColors)
else

*Make sure that the aColor array is set so you have something to save
  if ! valtype(aColor) == "A"
    W_Colors(.T.)
  endif
/*
This is where you put in your save array routine
*/
*  oLocal  := W_GetLocal()
*  oLocal..SysColor := aColor
*  oLocal:putRec()
endif

return aColor


function W_Str2Arr(cString, cDelimiter)
/*
Translate a delimited string to an array of strings

cString is the string to translate
cDelimiter is the delimiter string so we know where to break it

ex: W_Str2Arr("A,B,C", ",") would return {"A", "B", "C"}
*/
local aRetVal := {}
local nLocate

DEFAULT cDelimiter TO ";"
do while (nLocate := at(cDelimiter, cString)) > 0
  aadd(aRetVal, substr(cString, 1, nLocate - 1))
  cString := substr(cString, nLocate + 1)
enddo
aadd(aRetVal, cString)
return aRetVal



function W_Arr2Str(aString, cDelimiter)
/*
Translate an array of string to a delimited string
aString is the array of strings to concatenate
cDelimiter is the string to put in between each item

ex: W_Arr2Str({"A", "B", "C"}, ", ") would return "A, B, C"
*/
local cRetVal, nLen, nLoop

DEFAULT cDelimiter TO ";"
nLen := len(aString)
cRetVal := aString[1]
for nLoop := 2 to nLen
  cRetVal := cRetVal + cDelimiter + aString[nLoop]
next
return cRetVal


function W_Wait(lWaiting, cMessage)
/*
Either change the cursor to an hourglass (if a mouse is active) or post a text
"WAIT" message while a process that will take more than a fraction of a second
is executing. It will also optionally post a message box describing the action
that is going on.

One caution is that if you use it in a function that is called from a window
handler that also has setfocus/killfocus action, you may get yourself in a loop.
Just be careful of possible recursion.

lWaiting is used to increment/decrement the counter that is used to determine
  when to set/clear the wait message or cursor change. It is not necessary if
  you just need to post a new message during a wait.
cMessage is an optional message to be displayed.
*/
static lUseMouse
static nOpen := 0
static cScreen
static cColor
static oFWin
static nRow, nCol

local oOutput, oMsg

*If this is an increment/decrement wait call
if valtype(lWaiting) == "L"

*Set up statics if first time called
  if lUseMouse == NIL

*Is there a mouse active?
    lUseMouse := pw():isMouse() .and. pw():isTMR(PWTMR_MOUSE)

*What is the screen color?
    cColor := pvParseList(pw():getColorScheme():windowColor, 6)
    if empty(cColor)
      cColor := "*N/W"
    endif

*Find the max row and column
    nRow := pwMaxRow()
    nCol := pwMaxCol() + 1
  endif

*If it's an increment call
  if lWaiting
    ++nOpen

*We only need to change the cursor or display "WAIT" on the first increment
    if nOpen == 1
      if lUseMouse
        pw():mouseCursor(PMCURSOR_HOUR)
      else
        cScreen := savescreen(nRow, nCol - 6, nRow, nCol)
        oOutput := pw():setOutput(NIL)
        @ nRow, nCol - 6 say " WAIT " color cColor
        pw():setOutput(oOutput)
      endif
    endif

*If it's a decrement (and there's a wait indicator in progress)
  elseif nOpen > 0
    --nOpen

*We only need to change the cursor or remove "WAIT" on the last decrement
    if nOpen == 0
      if lUseMouse
        pw():mouseCursor(PMCURSOR_ARROW)
      else
        oOutput := pw():setOutput(NIL)
        restscreen(nRow, nCol - 6, nRow, nCol, cScreen)
        pw():setOutput(oOutput)
      endif

*Remove the mesage window (if any)
      MsgWnd()
    endif
  endif
endif

*Display the message window if requested and we're in a wait indication
if valtype(cMessage) == "C" .and. nOpen > 0
  MsgWnd(cMessage)
endif
return (nOpen > 0)



function MsgWnd(cMessage)
/*
Display a simple window for messaging. Since it gets used *a lot* we don't
destroy it in between calls.

cMessage is the message to display. If nothing passed, close the window
*/
local oPrev, nRow, nCol
static oMsgWin

*If we don't have a window, create it
if ! valtype(oMsgWin) == "O"
  CREATE WINDOW oMsgWin CENTERED;
    SIZE 5, 80;
    STYLE PWSTYLE_DIALOG - PWSTYLEINFO_TITLEBAR - PWSTYLEINFO_SYSTEMMENU
endif

if valtype(cMessage) == "C"

*Get the previous window
  oPrev := pw():curFocus()

*Find out its current position
  oMsgWin:GetPos(@nRow, @nCol)

*Let's not muck with the user's head by watching all the changes going on
  _pwDispBegin()

*Recenter it for the size of the current message
  oMsgWin:Open()
  oMsgWin:Move(nRow, int((pwMaxCol() - len(cMessage) - 3) / 2))
  oMsgWin:Size(5, len(cMessage) + 4)

  @ 1, 0 say " " + cMessage + " "

*Now we'll let them see the message
  _pwDispEnd()

*Go back to where we were, but make sure the message window stays where it can
*be seen
  if valtype(oPrev) == "O"
    pw():select(oPrev)
    _pwFront(oMsgWin:handle)
  endif
else

  oMsgWin:Close()
endif

return oMsgWin

